@@ -196,12 +196,12 @@ nautilus_emblem_verify_keyword (GtkWindow *parent_window,
 	if (keyword == NULL || strlen (keyword) == 0) {
 		eel_show_error_dialog (_("The emblem cannot be installed."),
 				       _("Sorry, but you must specify a non-blank keyword for the new emblem."), 
-				       _("Couldn't Install Emblem"), GTK_WINDOW (parent_window));
+				       GTK_WINDOW (parent_window));
 		return FALSE;
 	} else if (!emblem_keyword_valid (keyword)) {
 		eel_show_error_dialog (_("The emblem cannot be installed."),
 				       _("Sorry, but emblem keywords can only contain letters, spaces and numbers."), 
-				       _("Couldn't Install Emblem"), GTK_WINDOW (parent_window));
+				       GTK_WINDOW (parent_window));
 		return FALSE;
 	} else if (is_reserved_keyword (keyword)) {
 		char *error_string;
@@ -212,7 +212,7 @@ nautilus_emblem_verify_keyword (GtkWindow *parent_window,
 		 */
 		error_string = g_strdup_printf (_("Sorry, but there is already an emblem named \"%s\"."), display_name);
 		eel_show_error_dialog (_("Please choose a different emblem name."), error_string,
-				       _("Couldn't install emblem"), GTK_WINDOW (parent_window));
+				       GTK_WINDOW (parent_window));
 		g_free (error_string);
 		return FALSE;
 	} 
@@ -255,8 +255,7 @@ nautilus_emblem_install_custom_emblem (GdkPixbuf *pixbuf,
 	/* save the image */
 	if (eel_gdk_pixbuf_save_to_file (pixbuf, path) != TRUE) {
 		eel_show_error_dialog (_("The emblem cannot be installed."),
-				       _("Sorry, unable to save custom emblem."), 
-				       _("Couldn't Install Emblem"), 
+				       _("Sorry, unable to save custom emblem."), 				       
 				       GTK_WINDOW (parent_window));
 		g_free (dir);
 		g_free (stat_dir);
@@ -274,7 +273,7 @@ nautilus_emblem_install_custom_emblem (GdkPixbuf *pixbuf,
 		if (file == NULL) {
 			eel_show_error_dialog (_("The emblem cannot be installed."),
 					       _("Sorry, unable to save custom emblem name."), 
-					       _("Couldn't Install Emblem"), GTK_WINDOW (parent_window));
+					       GTK_WINDOW (parent_window));
 			g_free (stat_dir);
 			g_free (dir);
 			return;
