@@ -23,8 +23,12 @@
 */
 
 #include <gtk/gtktreemodel.h>
+#include <gtk/gtktreeview.h>
+#include <gdk/gdkdnd.h>
 #include <libnautilus-private/nautilus-file.h>
+#include <libnautilus-private/nautilus-directory.h>
 #include <libnautilus-private/nautilus-icon-factory.h>
+#include <libnautilus-extension/nautilus-column.h>
 
 #ifndef FM_LIST_MODEL_H
 #define FM_LIST_MODEL_H
@@ -37,6 +41,7 @@
 
 enum {
 	FM_LIST_MODEL_FILE_COLUMN,
+	FM_LIST_MODEL_SUBDIRECTORY_COLUMN,
 	FM_LIST_MODEL_SMALLEST_ICON_COLUMN,
 	FM_LIST_MODEL_SMALLER_ICON_COLUMN,
 	FM_LIST_MODEL_SMALL_ICON_COLUMN,
@@ -44,10 +49,13 @@ enum {
 	FM_LIST_MODEL_LARGE_ICON_COLUMN,
 	FM_LIST_MODEL_LARGER_ICON_COLUMN,
 	FM_LIST_MODEL_LARGEST_ICON_COLUMN,
-	FM_LIST_MODEL_NAME_COLUMN,
-	FM_LIST_MODEL_SIZE_COLUMN,
-	FM_LIST_MODEL_TYPE_COLUMN,
-	FM_LIST_MODEL_DATE_MODIFIED_COLUMN,
+	FM_LIST_MODEL_SMALLEST_EMBLEM_COLUMN,
+	FM_LIST_MODEL_SMALLER_EMBLEM_COLUMN,
+	FM_LIST_MODEL_SMALL_EMBLEM_COLUMN,
+	FM_LIST_MODEL_STANDARD_EMBLEM_COLUMN,
+	FM_LIST_MODEL_LARGE_EMBLEM_COLUMN,
+	FM_LIST_MODEL_LARGER_EMBLEM_COLUMN,
+	FM_LIST_MODEL_LARGEST_EMBLEM_COLUMN,
 	FM_LIST_MODEL_FILE_NAME_IS_EDITABLE_COLUMN,
 	FM_LIST_MODEL_NUM_COLUMNS
 };
@@ -61,27 +69,71 @@ typedef struct FMListModel {
 
 typedef struct {
 	GObjectClass parent_class;
+
+	void (* subdirectory_unloaded)(FMListModel *model,
+				       NautilusDirectory *subdirectory);
 } FMListModelClass;
 
 GType    fm_list_model_get_type                          (void);
-void     fm_list_model_add_file                          (FMListModel          *model,
-							  NautilusFile         *file);
+gboolean fm_list_model_add_file                          (FMListModel          *model,
+							  NautilusFile         *file,
+							  NautilusDirectory    *directory);
 void     fm_list_model_file_changed                      (FMListModel          *model,
-							  NautilusFile         *file);
+							  NautilusFile         *file,
+							  NautilusDirectory    *directory);
 gboolean fm_list_model_is_empty                          (FMListModel          *model);
+guint    fm_list_model_get_length                        (FMListModel          *model);
 void     fm_list_model_remove_file                       (FMListModel          *model,
-							  NautilusFile         *file);
+							  NautilusFile         *file,
+							  NautilusDirectory    *directory);
 void     fm_list_model_clear                             (FMListModel          *model);
 gboolean fm_list_model_get_tree_iter_from_file           (FMListModel          *model,
+							  NautilusFile         *file,
+							  NautilusDirectory    *directory,
+							  GtkTreeIter          *iter);
+GList *  fm_list_model_get_all_iters_for_file            (FMListModel          *model,
+							  NautilusFile         *file);
+gboolean fm_list_model_get_first_iter_for_file           (FMListModel          *model,
 							  NautilusFile         *file,
 							  GtkTreeIter          *iter);
 void     fm_list_model_set_should_sort_directories_first (FMListModel          *model,
 							  gboolean              sort_directories_first);
-int      fm_list_model_get_sort_column_id_from_attribute (const char           *attribute);
-int      fm_list_model_get_sort_column_id_from_sort_type (NautilusFileSortType  sort_type);
-char    *fm_list_model_get_attribute_from_sort_column_id (int                   sort_column_id);
+
+int      fm_list_model_get_sort_column_id_from_attribute (FMListModel *model,
+
+							  const char           *attribute);
+char    *fm_list_model_get_attribute_from_sort_column_id (FMListModel *model,
+							  int sort_column_id);
+void     fm_list_model_sort_files                        (FMListModel *model,
+							  GList **files);
 
 NautilusZoomLevel fm_list_model_get_zoom_level_from_column_id (int               column);
 int               fm_list_model_get_column_id_from_zoom_level (NautilusZoomLevel zoom_level);
+NautilusZoomLevel fm_list_model_get_zoom_level_from_emblem_column_id (int               column);
+int               fm_list_model_get_emblem_column_id_from_zoom_level (NautilusZoomLevel zoom_level);
 
+NautilusFile *    fm_list_model_file_for_path (FMListModel *model, GtkTreePath *path);
+gboolean          fm_list_model_load_subdirectory (FMListModel *model, GtkTreePath *path, NautilusDirectory **directory);
+void              fm_list_model_unload_subdirectory (FMListModel *model, GtkTreeIter *iter);
+
+void              fm_list_model_set_drag_view (FMListModel *model,
+					       GtkTreeView *view,
+					       int begin_x, 
+					       int begin_y);
+
+void              fm_list_model_get_drag_types (const GtkTargetEntry **entries,
+						int *num_entries);
+int               fm_list_model_compare_func (FMListModel *model,
+					      NautilusFile *file1,
+					      NautilusFile *file2);
+
+
+int               fm_list_model_add_column (FMListModel *model,
+					    NautilusColumn *column);
+int               fm_list_model_get_column_number (FMListModel *model,
+						   const char *column_name);
+						   
+void              fm_list_model_subdirectory_done_loading (FMListModel       *model,
+							   NautilusDirectory *directory);
+						   
 #endif /* FM_LIST_MODEL_H */
