@@ -39,6 +39,7 @@
 #include "nautilus-window-bookmarks.h"
 #include "nautilus-window-private.h"
 #include "nautilus-desktop-window.h"
+#include "nautilus-search-bar.h"
 #include <eel/eel-debug.h>
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-gnome-extensions.h>
@@ -65,11 +66,14 @@
 #include <libnautilus-private/nautilus-icon-factory.h>
 #include <libnautilus-private/nautilus-module.h>
 #include <libnautilus-private/nautilus-undo-manager.h>
+#include <libnautilus-private/nautilus-search-directory.h>
+#include <libnautilus-private/nautilus-search-engine.h>
 
 #define MENU_PATH_EXTENSION_ACTIONS                     "/MenuBar/File/Extension Actions"
 #define POPUP_PATH_EXTENSION_ACTIONS                     "/background/Before Zoom Items/Extension Actions"
 
-#define COMPUTER_URI          "computer:"
+#define NETWORK_URI          "network:"
+#define COMPUTER_URI         "computer:"
 #define BURN_CD_URI          "burn:"
 
 /* Struct that stores all the info necessary to activate a bookmark. */
@@ -220,8 +224,10 @@ action_connect_to_server_callback (GtkAction *action,
 {
 	NautilusWindow *window = NAUTILUS_WINDOW (user_data);
 	GtkWidget *dialog;
-	
-	dialog = nautilus_connect_server_dialog_new (window, nautilus_window_get_location (window));
+	char *location;
+	location = nautilus_window_get_location (window);
+	dialog = nautilus_connect_server_dialog_new (window, location);
+	g_free (location);
 
 	gtk_widget_show (dialog);
 }
@@ -274,6 +280,14 @@ action_go_to_computer_callback (GtkAction *action,
 			       COMPUTER_URI);
 }
 
+static void
+action_go_to_network_callback (GtkAction *action, 
+				gpointer user_data) 
+{
+	nautilus_window_go_to (NAUTILUS_WINDOW (user_data),
+			       NETWORK_URI);
+}
+
 static void
 action_go_to_templates_callback (GtkAction *action,
 				 gpointer user_data) 
@@ -664,7 +678,7 @@ static const GtkActionEntry main_entries[] = {
     N_("Connect to _Server..."), NULL,           /* label, accelerator */
     N_("Set up a connection to a network server"),                                      /* tooltip */ 
     G_CALLBACK (action_connect_to_server_callback) },
-  { "Home", GTK_STOCK_HOME,                        /* name, stock id */
+  { "Home", "gnome-fs-home",                        /* name, stock id */
     N_("_Home"), "<alt>Home",           /* label, accelerator */
     N_("Go to the home folder"),                                  /* tooltip */ 
     G_CALLBACK (action_home_callback) },
@@ -672,6 +686,10 @@ static const GtkActionEntry main_entries[] = {
     N_("_Computer"), NULL,           /* label, accelerator */
     N_("Go to the computer location"),                                  /* tooltip */ 
     G_CALLBACK (action_go_to_computer_callback) },
+  { "Go to Network", "gnome-fs-network",                        /* name, stock id */
+    N_("_Network"), NULL,           /* label, accelerator */
+    N_("Go to the network location"),                                  /* tooltip */ 
+    G_CALLBACK (action_go_to_network_callback) },
   { "Go to Templates", "stock_new-template",                        /* name, stock id */
     N_("T_emplates"), NULL,           /* label, accelerator */
     N_("Go to the templates folder"),                                  /* tooltip */ 
